package com.apobbati.samples.security;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.saml.SAMLCredential;
import org.springframework.security.saml.userdetails.SAMLUserDetailsService;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

class SAMLUserDetailsServiceImpl implements SAMLUserDetailsService {
    private Logger log = LoggerFactory.getLogger(SAMLUserDetailsServiceImpl.class);

    private Map<String, String> groupsToRolesMap;

    private String uidAttribute = "EmailAddress";

    private String groupsAttribute = "Groups";

    public String getUidAttribute() {
        return uidAttribute;
    }

    public void setUidAttribute(String uidAttribute) {
        this.uidAttribute = uidAttribute;
    }

    public String getGroupsAttribute() {
        return groupsAttribute;
    }

    public void setGroupsAttribute(String groupsAttribute) {
        this.groupsAttribute = groupsAttribute;
    }

    public Map<String, String> getGroupsToRolesMap() {
        return groupsToRolesMap;
    }

    public void setGroupsToRolesMap(Map<String, String> groupsToRolesMap) {
        this.groupsToRolesMap = groupsToRolesMap;
    }

    public Object loadUserBySAML(SAMLCredential samlCredential) throws UsernameNotFoundException {
        log.info("----------------------------------------");
        log.info("Building UserObject from SAML Credential: " + samlCredential);

        String memberOfAttr = SAMLUtil.getAttributeValue(samlCredential, groupsAttribute);
        String email = SAMLUtil.getAttributeValue(samlCredential, uidAttribute);

        if(samlCredential != null) {
            List<SimpleGrantedAuthority> grantedAuthorities = new ArrayList<SimpleGrantedAuthority>();

            if(memberOfAttr != null) {
                for(String group : memberOfAttr.split(",")) {
                    String mappedRole = groupsToRolesMap == null ? group : groupsToRolesMap.get(group);
                    if(mappedRole == null)
                        mappedRole = group;
                    grantedAuthorities.add(new SimpleGrantedAuthority(mappedRole));
                }
            }

            Map<String, String> attributes = new HashMap<>();
            attributes.put("email", SAMLUtil.getAttributeValue(samlCredential, "EmailAddress"));
            attributes.put("name", SAMLUtil.getAttributeValue(samlCredential, "FullName"));
            attributes.put("uid", samlCredential.getNameID().getValue());

            return new CustomUserDetails(email, "", grantedAuthorities, attributes);
        }

        log.error("The SAMLCredential was null!");
        return null;
    }
}
